/*****************************************************************************************
 * Copyright (C), 2008
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   NAME:    orva_simulate.sas                                            
 *   PURPOSE: Wrapper for %orva_loss_sim and %orva_conf_interval macros.
 *   SUPPORT: stchri            
 *   
 *   PARAMETERS: see documentation for %orva_loss_sim macro.
 ***********************************************************************************/
%macro orva_simulate(
                     sParmPriorityData  =  , 
                     fParmPriorityData  =  , 
                     combineMethod      =  ,
                     insuranceMethod    =  BASEL,
                     insuranceEffect    =  ,
                     lossKriData        =  ,
                     lossKriDataC       =  ,
                     freqSimData        =  ,
                     ccUid              =  ,
                     ccData             =  ,
                     seedData           =  ,
                     outsimNaked        =  ,     
                     outsimInsured      =  ,
                     outstatNaked       =  ,   
                     outstatInsured     =  ,
                     ConfIntData        =  ,
                     SimInterval        =  ,  /* this is not used anywhere */
                     parallel           =  0,
                     nstreams           =  1,
                     cpiAdjustment      =  1.0,
                     rareScenSim        =  ,
                     selectedScenUids   =  ,
                     lookupSize         =  1000
                     );
   %if(%upcase(&ORV_DEBUG.) EQ TRUE) %then %do;
      %put #orva_simulate(;
      %put .              sParmPriorityData  =  &sParmPriorityData,; 
      %put .              fParmPriorityData  =  &fParmPriorityData,;
      %put .              combineMethod      =  &combineMethod,;
      %put .              insuranceMethod    =  &insuranceMethod,;
      %put .              insuranceEffect    =  &insuranceEffect,;
      %put .              lossKriData        =  &lossKriData,;
      %put .              lossKriDataC       =  &lossKriDataC,;
      %put .              freqSimData        =  &freqSimData,;
      %put .              ccUid              =  &ccUid,;
      %put .              ccData             =  &ccData,;
      %put .              seedData           =  &seedData,;
      %put .              outsimNaked        =  &outsimNaked,;     
      %put .              outsimInsured      =  &outsimInsured,;
      %put .              outstatNaked       =  &outstatNaked,;  
      %put .              outstatInsured     =  &outstatInsured,;
      %put .              ConfIntData        =  &ConfIntData,;
      %put .              SimInterval        =  &SimInterval,;
      %put .              parallel           =  &parallel,;
      %put .              nstreams           =  &nstreams,;
      %put .              cpiAdjustment      =  &cpiAdjustment,;
      %put .              rareScenSim        =  &rareScenSim,;
      %put .              selectedScenUids   =  &selectedScenUids,;
      %put .              lookupSize         =  &lookupSize;
      %put .              );
   %end;

   /*--- get SEED from seedData ---*/
   %let SEED = 0;

   data _NULL_; 
      set &seedData; 
      WHERE 
            CCUID eq &ccUID
            ;
      length tseed $ 32;
      tseed = put(seed, 32.);
      call symput("SEED", trim(left(tseed)));
   run;
  
   /*--- Extract MAXLOSS from sParmPriorityData ---*/
   %let MAXLOSS = 10000000000;
   data _NULL_; 
      SET &sParmPriorityData end = LAST; 
      WHERE 
            CCUID = &ccUID 
      AND 
            SOURCE_TYPE NE "R"
            ;
      retain MaximumLoss 0;
      MaximumLoss = max(MaximumLoss, max_loss);

      if LAST then do;
         if MaximumLoss eq 0 then do;
            MaximumLoss = 10000000000;
         end;

         call symputX("MAXLOSS", put(MaximumLoss, BEST20.));
      end;
   run;

   %let ORV_ABORT = &ORV_FALSE;
   %orva_insurance_get_effect(
                              CCUid              = &ccUid,
                              insuranceEffect    = &insuranceEffect,
                              insuranceContracts = VARPRJ.ins_contracts
                              );
   %if &ORV_ABORT %then %do;
      %let ORV_REASON = Insurance effect calculation cannot be completed;
      %put WARNING:&ORV_REASON.;     
   %end;  
   
   /*--- perform loss simulation ---*/
   %orva_loss_sim(
                  sParmPriorityData =  &sParmPriorityData,  
                  combineMethod     =  &combineMethod,     
                  insuranceMethod   =  &insuranceMethod,
                  lossKriData       =  &lossKriData,
                  lossKriDataC      =  &lossKriDataC,
                  freqSimData       =  &freqSimData,
                  ccUid             =  &ccUid,
                  ccData            =  &ccData,
                  seedData          =  &seedData,
                  lookupSize        =  &lookupSize,
                  outsimNaked       =  _tmpSimNaked,     
                  outsimInsured     =  _tmpSimInsured,
                  outstatNaked      =  _tmpStatNaked,   
                  outstatInsured    =  _tmpStatInsured,
                  confIntData       =  &confIntData,
                  parallel          =  &parallel,
                  nstreams          =  &nstreams,
                  cpiAdjustment     =  &cpiAdjustment,
                  rareScenSim       =  &rareScenSim,
                  selectedScenUids  =  &selectedScenUids
                  );

   /*--- make sure loss simulation was successful ---*/
   %let status = 1;

   data _NULL_; 
      set _tmpStatNaked;
      call symputX("status", status);
      call symput("ORV_REASON", reason);
   run;
     
   /****** GOOD results from simulation ******/
   %if &status eq 0 %then %do;
      %orv_createSimResultDatasets();

      %orva_var_sensitivities( 
                               ccUid              = &ccUid.,
                               selectedScenUids   = &selectedScenUids.,
                               SimPriority_Sev    = &sParmPriorityData.,  
                               SimPriority_Freq   = &fParmPriorityData.,  
                               rareScenSim        = &rareScenSim. ,
                               lossKriData        = &lossKriData.,
                               lossKriDataC       = &lossKriDataC., 
                               tempStatNaked      = _tmpStatNaked,
                               tempStatInsured    = _tmpStatInsured,
                               SimInterval        = &SimInterval.,
                               maxloss            = &maxloss.
                               );

      /*--- append resultant datasets to output datasets ---*/
      %orvu_append(
                   base         = &outsimNaked,    
                   new          = _tmpSimNaked,    
                   canUseAppend = false
                   );
      %orvu_append(
                   base         = &outsimInsured,  
                   new          = _tmpSimInsured,  
                   canUseAppend = false
                   );
      %orvu_append(
                   base         = &outstatNaked,   
                   new          = _tmpStatNaked,   
                   canUseAppend = false
                   );
      %orvu_append(
                   base         = &outstatInsured, 
                   new          = _tmpStatInsured, 
                   canUseAppend = false
                   );
   %end;
   /****** BAD results from simulation ******/
   %else %do;
%DONE_NO_INSURANCE:
      data _tmpStatNaked;
         format reason &ORVAR_REASON_FMT.;
         status = &status;
         reason = "&ORV_REASON";
         CCUID  = &ccUid; 
      run;

      %orvu_append(
                   base         = &outstatNaked,   
                   new          = _tmpStatNaked, 
                   canUseAppend = false
                   );
      %orvu_append(
                   base         = &outstatInsured, 
                   new          = _tmpStatNaked, 
                   canUseAppend = false
                   );     
   %end;

%DONE_SIMULATE:
   %orvu_delete_data( data = 
                              _tmpSimNaked 
                              _tmpSimInsured 
                              _tmpStatNaked 
                              _tmpStatInsured
                              _sim_:
                              );
%mend orva_simulate;

%macro orv_createSimResultDatasets();
        data _tmpSimNaked; 
           set _tmpSimNaked; 
           CCUID = &ccUid; 
        run;

        data _tmpSimInsured; 
           set _tmpSimInsured; 
           CCUID = &ccUid;
        run;

        data _tmpStatNaked; 
           set _tmpStatNaked; 
           drop _TYPE_;
           CCUID = &ccUid; 
        run;

        data _tmpStatInsured; 
           set _tmpStatInsured;
           drop _TYPE_;
           CCUID = &ccUid;
        run;
%mend orv_createSimResultDatasets;   
